const { readFileSync } = require('fs');
const path = require('path');
const requireResolve = require('require-resolve');

const { defaultOptions } = require('./consts');

/**
 * Visitor for `import '*.css'` babel AST-nodes
 */
function CssImport(cb) {
  return ({ node, replaceWith }, { file, opts = {} }) => {
    if (!node.source.value.endsWith('.css')) return;

    const { src } = requireResolve(node.source.value, path.resolve(file.opts.filename));
    const cssContent = readFileSync(src).toString();

    const options = { ...defaultOptions, ...opts };

    cb({
      src,
      css: cssContent,
      replaceWith,
      importNode: { ...node, ...node.specifiers[0] },
      options,
    });
  };
}

module.exports = CssImport;
